import builtins
import typing

class FieldIndex: ...

class FieldSpec:
    def required(self) -> FieldSpec: ...
    def optional(self) -> FieldSpec: ...
    def index(self, index: FieldIndex) -> FieldSpec: ...

# data types
def text() -> FieldSpec: ...
def int() -> FieldSpec: ...
def float() -> FieldSpec: ...
def bool() -> FieldSpec: ...
def f32_vector(dimension: builtins.int) -> FieldSpec: ...
def u8_vector(dimension: builtins.int) -> FieldSpec: ...
def binary_vector(dimension: builtins.int) -> FieldSpec: ...
def f32_sparse_vector() -> FieldSpec: ...
def u8_sparse_vector() -> FieldSpec: ...
def bytes() -> FieldSpec: ...

# indexes
def vector_index(
    metric: typing.Literal["cosine", "euclidean", "dot_product", "hamming"],
) -> FieldIndex: ...
def keyword_index() -> FieldIndex: ...
def semantic_index(model: str) -> FieldIndex: ...



import builtins
import typing
from enum import Enum

from . import data, error, query, schema  # noqa

class Client:
    def __new__(
        cls,
        api_key: builtins.str,
        region: builtins.str,
        host: builtins.str = "topk.io",
        https: builtins.bool = True,
        retry_config: typing.Optional[RetryConfig] = None,
    ) -> Client: ...
    def collection(self, collection: builtins.str) -> CollectionClient: ...
    def collections(self) -> CollectionsClient: ...

class Collection:
    name: builtins.str
    org_id: builtins.str
    project_id: builtins.str
    region: builtins.str
    schema: builtins.dict[builtins.str, schema.FieldSpec]
    def __new__(
        cls,
        name: builtins.str,
        org_id: builtins.str,
        project_id: builtins.str,
        region: builtins.str,
        schema: typing.Mapping[builtins.str, schema.FieldSpec],
    ) -> Collection: ...
    def __repr__(self) -> builtins.str: ...
    def __eq__(self, other: Collection) -> builtins.bool: ...

class CollectionClient:
    def get(
        self,
        ids: typing.Sequence[builtins.str],
        fields: typing.Optional[typing.Sequence[builtins.str]] = None,
        lsn: typing.Optional[builtins.str] = None,
        consistency: typing.Optional[ConsistencyLevel] = None,
    ) -> builtins.dict[builtins.str, builtins.dict[builtins.str, typing.Any]]: ...
    def count(
        self,
        lsn: typing.Optional[builtins.str] = None,
        consistency: typing.Optional[ConsistencyLevel] = None,
    ) -> builtins.int: ...
    def query(
        self,
        query: query.Query,
        lsn: typing.Optional[builtins.str] = None,
        consistency: typing.Optional[ConsistencyLevel] = None,
    ) -> builtins.list[builtins.dict[builtins.str, typing.Any]]: ...
    def upsert(
        self, documents: typing.Sequence[typing.Mapping[builtins.str, typing.Any]]
    ) -> builtins.str: ...
    def delete(self, ids: typing.Sequence[builtins.str]) -> builtins.str: ...

class CollectionsClient:
    def get(self, collection_name: builtins.str) -> Collection: ...
    def list(self) -> builtins.list[Collection]: ...
    def create(
        self,
        collection_name: builtins.str,
        schema: typing.Mapping[builtins.str, schema.FieldSpec],
    ) -> Collection: ...
    def delete(self, collection_name: builtins.str) -> None: ...

class RetryConfig: ...

class ConsistencyLevel(Enum):
    Indexed = ...
    Strong = ...



class Value: ...
class Vector: ...

def f32_vector(data: list[float]) -> Value: ...
def u8_vector(data: list[int]) -> Value: ...
def binary_vector(data: list[int]) -> Value: ...
def f32_sparse_vector(data: dict[int, float]) -> Value: ...
def u8_sparse_vector(data: dict[int, int]) -> Value: ...



import pytest
from topk_sdk.data import binary_vector, f32_vector, u8_vector

TYPE_ERROR = "Invalid vector value"


class TestF32Vector:
    def test_valid(self):
        f32_vector([1, 2, 3])

    def test_empty_case(self):
        f32_vector([])

    def test_to_string(self):
        assert str(f32_vector([1, 2, 3])) == "Vector(F32([1.0, 2.0, 3.0]))"

    def test_invalid_arguments(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector(0)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector(None)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector(False)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector(float("nan"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector(float("inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector(float("-inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector({1: 256})  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_vector({1: -1})  # type: ignore


class TestU8Vector:
    def test_valid(self):
        u8_vector([1, 2, 3])

    def test_empty_case(self):
        u8_vector([])

    def test_to_string(self):
        assert str(u8_vector([1, 2, 3])) == "Vector(U8([1, 2, 3]))"

    def test_invalid_number_range(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector([256])  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector([-1])  # type: ignore

    def test_invalid_arguments(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector(0)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector(None)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector(False)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector(float("nan"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector(float("inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector({1: 256})  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_vector({1: -1})  # type: ignore


class TestBinaryVector:
    def test_valid(self):
        binary_vector([1, 2, 3])

    def test_empty_case(self):
        binary_vector([])

    def test_to_string(self):
        assert str(binary_vector([1, 2, 3])) == "Vector(U8([1, 2, 3]))"

    def test_invalid_number_range(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector([256])  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector([-1])  # type: ignore

    def test_invalid_arguments(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector(0)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector(None)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector(False)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector(float("nan"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector(float("inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector(float("-inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector({1: 256})  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            binary_vector({1: -1})  # type: ignore



import pytest
from topk_sdk.data import f32_sparse_vector, u8_sparse_vector

TYPE_ERROR = "Invalid sparse vector"


class TestF32SparseVector:
    def test_valid(self):
        f32_sparse_vector({1: 1.1})
        f32_sparse_vector({1: 1})

    def test_empty_case(self):
        f32_sparse_vector({})

    def test_to_string(self):
        assert str(f32_sparse_vector({1: 1.1})) == "SparseVector(F32([1], [1.1]))"

    def test_invalid_key(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector({"foo": 1})  # type: ignore

    def test_invalid_value(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector({1: "foo"})  # type: ignore

    def test_invalid_arguments(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector(0)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector([])  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector(None)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector(False)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector(float("nan"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector(float("inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            f32_sparse_vector(float("-inf"))  # type: ignore


class TestU8SparseVector:
    def test_valid(self):
        u8_sparse_vector({1: 1})
        u8_sparse_vector({1: 1, 2: 2})

    def test_empty_case(self):
        u8_sparse_vector({})

    def test_to_string(self):
        assert str(u8_sparse_vector({1: 1})) == "SparseVector(U8([1], [1]))"

    def test_invalid_key(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector({"foo": 1})  # type: ignore

    def test_invalid_value(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector({1: "foo"})  # type: ignore

    def test_invalid_number_range(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector({1: 256})  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector({1: -1})  # type: ignore

    def test_invalid_arguments(self):
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector(0)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector([])  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector(None)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector(False)  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector(float("nan"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector(float("inf"))  # type: ignore
        with pytest.raises(TypeError, match=TYPE_ERROR):
            u8_sparse_vector(float("-inf"))  # type: ignore



from typing import Any


def doc_ids(result: list[dict[str, Any]]) -> set[str]:
    return {doc["_id"] for doc in result}


def doc_ids_ordered(result: list[dict[str, Any]]) -> list[str]:
    return [doc["_id"] for doc in result]
